/*------------------------------------------------------------------------------
| Module:      wine_perl_gateway.c                                             |
| ---------------------------------------------------------------------------- |
| Purpose:     Gateway functions to allow access to wine API functions from    |
|              perl                                                            |
|                                                                              |
| Functions:                                                                   |
|                                                                              |
|     perl_init_wine                                                           |
|     perl_call_wine                                                           |
|                                                                              |
------------------------------------------------------------------------------*/



/*----------------------------------------------------------------------
| Preprocessor stuff                                                   |
----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "winbase.h"
#include "process.h"

#include "wine_perl_gateway.h"

#define str_eq(s1, s2)    (strcmp ((s1), (s2)) == 0)



/*----------------------------------------------------------------------
| Local prototypes                                                     |
----------------------------------------------------------------------*/
void PROCESS_InitWinelib
(
    int   argc,
    char  **argv
);

static unsigned long call_wine_func 
(
    FARPROC        proc,
    int            n_args,
    unsigned long  *a
);



/*----------------------------------------------------------------------
| Function:    perl_init_wine                                          |
| -------------------------------------------------------------------- |
| Purpose:     Call wine initialization function                       |
|                                                                      |
| Parameters:  argc, argv                                              |
|                                                                      |
| Returns:                                                             |
----------------------------------------------------------------------*/
int perl_init_wine (int argc, char *argv[])
{

/* Begin perl_init_wine */

    PROCESS_InitWinelib (argc, argv);

    return (0);

} /* End perl_init_wine */



/*----------------------------------------------------------------------
| Function:    perl_call_wine                                          |
| -------------------------------------------------------------------- |
| Purpose:     Fetch and call a wine API function from a library       |
|                                                                      |
| Parameters:                                                          |
|                                                                      |
|     module    -- module in function (ostensibly) resides             |
|     function  -- function name                                       |
|     n_args    -- number of args                                      |
|     args      -- args                                                |
|     debug     -- debug flag                                          |
|                                                                      |
| Returns:     Return value from API function called                   |
----------------------------------------------------------------------*/
unsigned long perl_call_wine 
(
    char           *module,
    char           *function,
    int            n_args,
    unsigned long  *args,
    int            debug
)
{
    /* Locals */
    HMODULE        hmod;
    FARPROC        proc;
    int            i;

    unsigned long  r;

/* Begin perl_call_wine */

    /*--------------------------------------------------------------
    | Debug
    --------------------------------------------------------------*/
    if (debug)
    {
        printf ("    [gtwy.c/perl_call_wine()]\n");
        for (i = 0; (i < n_args); i++)
        {
            printf ("        0x%lx\n", args[i]);
        }
    }

    /*--------------------------------------------------------------
    | See if we can load specified module
    --------------------------------------------------------------*/
    if ((hmod = LoadLibraryA (module)) == NULL)
    {
        fprintf (stderr, "    LoadLibraryA(%s) failed\n", module);
        goto EXIT;
    }

    /*--------------------------------------------------------------
    | See if we can get address of specified function from it
    --------------------------------------------------------------*/
    if ((proc = GetProcAddress (hmod, function)) == NULL)
    {
        fprintf (stderr, "    GetProcAddress(%s.%s) failed\n", module, function);
        goto EXIT;
    }

    /*--------------------------------------------------------------
    | Righty then; call the function ...
    --------------------------------------------------------------*/
    r = call_wine_func (proc, n_args, args);


    /*--------------------------------------------------------------
    | Buh-bye
    --------------------------------------------------------------*/
EXIT:

    return (r);

} /* End perl_call_wine */



/*----------------------------------------------------------------------
| Function:    call_wine_func                                          |
| -------------------------------------------------------------------- |
| Purpose:     Call a wine API function, passing in appropriate number |
|              of args                                                 |
|                                                                      |
| Parameters:  proc   -- function to call                              |
|              n_args -- array of args                                 |
|              a      -- array of args                                 |
|                                                                      |
| Returns:     return value from API function called                   |
----------------------------------------------------------------------*/
static unsigned long call_wine_func 
(
    FARPROC        proc,
    int            n_args,
    unsigned long  *a
)
{
    /* Locals */
    unsigned long  rc;

/* Begin call_wine_func */

    /*--------------------------------------------------------------
    | Now we need to call the function with the appropriate number
    | of arguments
    |
    | Anyone who can think of a better way to do this is welcome to
    | come forth with it ...
    --------------------------------------------------------------*/
    switch (n_args)
    {

        case 0:  rc = proc ();                                                     break;
        case 1:  rc = proc (a[0]);                                                 break;
        case 2:  rc = proc (a[0], a[1]);                                           break;
        case 3:  rc = proc (a[0], a[1], a[2]);                                     break;
        case 4:  rc = proc (a[0], a[1], a[2], a[3]);                               break;
        case 5:  rc = proc (a[0], a[1], a[2], a[3], a[4]);                         break;
        case 6:  rc = proc (a[0], a[1], a[2], a[3], a[4], a[5]);                   break;
        case 7:  rc = proc (a[0], a[1], a[2], a[3], a[4], a[5], a[6]);             break;
        case 8:  rc = proc (a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7]);       break;
        case 9:  rc = proc (a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8]); break;

        case 10:
            rc = proc 
            (
                a[0],
                a[1],
                a[2],
                a[3],
                a[4],
                a[5],
                a[6],
                a[7],
                a[8],
                a[9]
            );
            break;

        case 11:
            rc = proc 
            (
                a[0],
                a[1],
                a[2],
                a[3],
                a[4],
                a[5],
                a[6],
                a[7],
                a[8],
                a[9],
                a[10]
            );
            break;

        case 12:
            rc = proc 
            (
                a[0],
                a[1],
                a[2],
                a[3],
                a[4],
                a[5],
                a[6],
                a[7],
                a[8],
                a[9],
                a[10],
                a[11]
            );
            break;

        case 13:
            rc = proc 
            (
                a[0],
                a[1],
                a[2],
                a[3],
                a[4],
                a[5],
                a[6],
                a[7],
                a[8],
                a[9],
                a[10],
                a[11],
                a[12]
            );
            break;

        case 14:
            rc = proc 
            (
                a[0],
                a[1],
                a[2],
                a[3],
                a[4],
                a[5],
                a[6],
                a[7],
                a[8],
                a[9],
                a[10],
                a[11],
                a[12],
                a[13]
            );
            break;

        case 15:
            rc = proc 
            (
                a[0],
                a[1],
                a[2],
                a[3],
                a[4],
                a[5],
                a[6],
                a[7],
                a[8],
                a[9],
                a[10],
                a[11],
                a[12],
                a[13],
                a[14]
            );
            break;

        case 16:
            rc = proc 
            (
                a[0],
                a[1],
                a[2],
                a[3],
                a[4],
                a[5],
                a[6],
                a[7],
                a[8],
                a[9],
                a[10],
                a[11],
                a[12],
                a[13],
                a[14],
                a[15]
            );
            break;

    }  /* end switch */


    /*--------------------------------------------------------------
    | Return value from func
    --------------------------------------------------------------*/
    return (rc);

} /* End call_wine_func */
